@@ -111,7 +111,7 @@ caddr_t TypeRef::mem(caddr_t addr)
     return addr;
 }
 
-stringref::value::value(caddr_t addr, size_t objsize, const char *str) : 
+typeref<const char *>::value::value(caddr_t addr, size_t objsize, const char *str) : 
 TypeRef::Counted(addr, objsize)
 {
     if(str)
@@ -120,19 +120,19 @@ TypeRef::Counted(addr, objsize)
 	    mem[0] = 0;
 }
 
-void stringref::value::destroy(void) 
+void typeref<const char *>::value::destroy(void) 
 {
 	count.clear();
 	release();
 }
 
-stringref::stringref() :
+typeref<const char *>::typeref() :
 TypeRef() {}
 
-stringref::stringref(const stringref& copy) :
+typeref<const char *>::typeref(const typeref<const char *>& copy) :
 TypeRef(copy) {}
 
-stringref::stringref(const char *str) :
+typeref<const char *>::typeref(const char *str) :
 TypeRef()
 {
     size_t size = 0;
@@ -144,7 +144,7 @@ TypeRef()
     TypeRef::set(new(mem(p)) value(p, size, str));
 }
 
-const char *stringref::operator()(ssize_t offset) const
+const char *typeref<const char *>::operator()(ssize_t offset) const
 {
     value *v = polystatic_cast<value *>(ref);
     if(!v)
@@ -162,7 +162,7 @@ const char *stringref::operator()(ssize_t offset) const
     return &v->mem[v->len() + offset];
 }
 
-const char *stringref::operator*() const 
+const char *typeref<const char *>::operator*() const 
 {
     if(!ref)
 	    return NULL;
@@ -170,19 +170,19 @@ const char *stringref::operator*() const
     return &v->mem[0];
 }
 
-stringref& stringref::operator=(const stringref& objref)
+typeref<const char *>& typeref<const char *>::operator=(const typeref<const char *>& objref)
 {
     TypeRef::set(objref);
     return *this;
 }
 
-stringref& stringref::operator=(const char *str)
+typeref<const char *>& typeref<const char *>::operator=(const char *str)
 {
     set(str);
     return *this;
 }
 
-void stringref::set(const char *str)
+void typeref<const char *>::set(const char *str)
 {
     release();
     size_t size = 0;
@@ -194,75 +194,121 @@ void stringref::set(const char *str)
     TypeRef::set(new(mem(p)) value(p, size, str));
 }
 
-void stringref::assign(value *chars)
+void typeref<const char *>::assign(value *chars)
 {
     release();
     chars->size = strlen(chars->mem);
     TypeRef::set(chars);
 }
 
-stringref& stringref::operator=(value *chars)
+typeref<const char *>& typeref<const char *>::operator=(value *chars)
 {
     assign(chars);
     return *this;
 }
 
-stringref::value *stringref::create(size_t size)
+typeref<const char *>::value *typeref<const char *>::create(size_t size)
 {
     caddr_t p = TypeRef::alloc(sizeof(value) + size);
     return new(mem(p)) value(p, size, NULL);
 }
 
-void stringref::destroy(stringref::value *chars)
+void typeref<const char *>::destroy(typeref<const char *>::value *chars)
 {
     if(chars)
         chars->destroy();
 }
 
-void stringref::expand(stringref::value **handle, size_t size)
+void typeref<const char *>::expand(typeref<const char *>::value **handle, size_t size)
 {
     if(!handle || !*handle)
         return;
 
-    stringref::value *change = create(size + (*handle)->max());
+    typeref<const char *>::value *change = create(size + (*handle)->max());
     if(change)
         String::set(change->get(), change->max() + 1, (*handle)->get());
     destroy(*handle);
     *handle = change;
 }
 
-byteref::value::value(caddr_t addr, size_t objsize, const uint8_t *str) : 
+bool typeref<const char *>::operator==(const typeref<const char *>& ptr) const
+{
+    value *v1 = polystatic_cast<value*>(ref);
+    value *v2 = polystatic_cast<value*>(ptr.ref);
+    if(!v1 || !v2)
+        return false;
+    return eq(&(v1->mem[0]), &(v2->mem[0]));
+}
+
+bool typeref<const char *>::operator==(const char *obj) const
+{
+    value *v = polystatic_cast<value *>(ref);
+    if(!v)
+        return false;
+    return eq(&(v->mem[0]), obj);
+}
+
+bool typeref<const char *>::operator==(value *chars) const
+{
+    value *v = polystatic_cast<value *>(ref);
+    if(!v || !chars)
+        return false;
+    return eq(&(v->mem[0]), &(chars->mem[0]));
+}
+
+bool typeref<const char *>::operator<(const typeref<const char *>& ptr) const
+{
+    value *v1 = polystatic_cast<value *>(ref);
+    value *v2 = polystatic_cast<value *>(ptr.ref);
+
+    if(!v1 && v2)
+        return true;
+
+    if(!v1 && !v2)
+        return true;
+
+    if(v1 && !v2)
+        return false;
+
+#ifdef  HAVE_STRCOLL
+    return strcoll(&(v1->mem[0]), &(v2->mem[0])) < 0;
+#else
+    return strcmp(&(v1->mem[0]), &(v2->mem[0])) < 0:
+#endif
+}
+    
+typeref<const uint8_t *>::value::value(caddr_t addr, size_t objsize, const uint8_t *str) : 
 TypeRef::Counted(addr, objsize)
 {
     if(objsize)
         memcpy(mem, str, objsize);
 }
 
-byteref::value::value(caddr_t addr, size_t size) : 
+typeref<const uint8_t *>::value::value(caddr_t addr, size_t size) : 
 TypeRef::Counted(addr, size)
 {
 }
 
-byteref::byteref() :
+typeref<const uint8_t *>::typeref() :
 TypeRef() {}
 
-byteref::byteref(const byteref& copy) :
+typeref<const uint8_t *>::typeref(const typeref<const uint8_t *>& copy) :
 TypeRef(copy) {}
 
-byteref::byteref(const uint8_t *str, size_t size) :
+typeref<const uint8_t *>::typeref(uint8_t *str, size_t size) :
 TypeRef()
 {
     caddr_t p = TypeRef::alloc(sizeof(value) + size);
     TypeRef::set(new(mem(p)) value(p, size, str));
 }
 
-void byteref::value::destroy(void) 
+void typeref<const uint8_t *>::value::destroy(void) 
 {
 	count.clear();
 	release();
 }
 
-const uint8_t *byteref::operator*() const 
+const uint8_t *typeref<const uint8_t *>::operator*() const 
 {
     if(!ref)
 	    return NULL;
@@ -270,41 +316,73 @@ const uint8_t *byteref::operator*() const
     return &v->mem[0];
 }
 
-byteref& byteref::operator=(const byteref& objref)
+typeref<const uint8_t *>& typeref<const uint8_t *>::operator=(const byteref& objref)
 {
     TypeRef::set(objref);
     return *this;
 }
 
-byteref& byteref::operator=(value *bytes)
+typeref<const uint8_t *>& typeref<const uint8_t *>::operator=(value *bytes)
 {
     assign(bytes);
     return *this;
 }
 
-void byteref::set(const uint8_t *str, size_t size)
+void typeref<const uint8_t *>::set(const uint8_t *str, size_t size)
 {
     release();
     caddr_t p = TypeRef::alloc(sizeof(value) + size);
     TypeRef::set(new(mem(p)) value(p, size, str));
 }
 
-void byteref::assign(value *bytes)
+void typeref<const uint8_t *>::assign(value *bytes)
 {
     release();
     TypeRef::set(bytes);
 }
 
-byteref::value *byteref::create(size_t size)
+typeref<const uint8_t *>::value *typeref<const uint8_t *>::create(size_t size)
 {
     caddr_t p = TypeRef::alloc(sizeof(value) + size);
     return new(mem(p)) value(p, size);
 }
 
-void byteref::destroy(byteref::value *bytes)
+void typeref<const uint8_t *>::destroy(typeref<const uint8_t *>::value *bytes)
 {
     if(bytes)
         bytes->destroy();
 }
 
+size_t TypeRef::size(void) const
+{
+    if(!ref)
+        return 0;
+
+    return ref->size;
+}
+
+unsigned TypeRef::copies() const 
+{
+	if(!ref)
+		return 0;
+	return ref->copies();
+}
+
+bool typeref<const uint8_t *>::operator==(const typeref<const uint8_t *>& ptr) const 
+{
+    value *v1 = polystatic_cast<value*>(ref);
+    value *v2 = polystatic_cast<value*>(ptr.ref);
+    if(!v1 || !v2 || v1->size != v2->size)
+        return false;
+    return !memcmp(&(v1->mem[0]), &(v2->mem[0]), v1->size);
+}
+
+bool typeref<const uint8_t *>::operator==(value *bytes) const 
+{
+    value *v = polystatic_cast<value *>(ref);
+    if(!v || !bytes || v->size != bytes->size)
+        return false;
+    return !memcmp(&(v->mem[0]), &(bytes->mem[0]), v->size);
+}
+
 } // namespace
