@@ -267,61 +267,52 @@ public:
 template <typename T>
 class temporary
 {
+private:
+    inline temporary(const temporary<T>&) {};
+
 protected:
-    T *object;
+    T *array;
+    size_t used;
+
 public:
     /**
      * Construct a temporary object, create our stack frame reference.
      */
-    inline temporary() {
-        object = NULL;
+    inline temporary(size_t size = 1) {
+        array = new T[size];
+        used = size;
+    }    
+
+    inline temporary(size_t size, const T initial) {
+        array = new T[size];
+        used = size;
+        for(size_t p = 0; p < size; ++p)
+            array[p] = initial;
     }
 
-    /**
-     * Disable copy constructor.
-     */
-    temporary(const temporary<T>&) {
-        ::abort();
+    inline explicit temporary(const T initial) {
+        array = new T[1];
+        used = 1;
+        array[0] = initial;
     }
 
-    /**
-     * Construct an assigned pointer.
-     */
-    inline temporary(T *ptr) {
-        object = ptr;
+    inline ~temporary() {
+        if(array) {
+            delete[] array;
+            array = NULL;
+        }
     }
 
-    /**
-     * Assign a temporary object.  This adds a pointer to an existing
-     * type to the current temporary pointer.  If the temporary was
-     * already assigned, then it is deleted.
-     * @param temp object to assign.
-     */
-    inline T& operator=(T *temp) {
-        if(object)
-            delete object;
-        object = temp;
-        return *this;
-    }
-
-    /**
-     * Assign a temporary object.  This adds a pointer to an existing
-     * type to the current temporary pointer.  If the temporary was
-     * already assigned, then it is deleted.
-     * @param temp object to assign.
-     */
-    inline void set(T *temp) {
-        if(object)
-            delete object;
-        object = temp;
-    }
+    inline operator T&() const {
+        return array[0];
+    }        
 
     /**
      * Access heap object through our temporary directly.
      * @return reference to heap resident object.
      */
     inline T& operator*() const {
-        return *object;
+        return array[0];
     }
 
     /**
@@ -329,6 +320,87 @@ public:
      * @return member reference of heap object.
      */
     inline T* operator->() const {
+        return &array[0];
+    }
+
+    inline operator bool() const {
+        return array != NULL;
+    }
+
+    inline bool operator!() const {
+        return array == NULL;
+    }
+
+    inline temporary& operator=(const T initial) {
+        array[0] = initial;
+        return *this;
+    }
+
+    inline void release() {
+        if(array) {
+            delete[] array;
+            array = NULL;
+        }
+    }
+
+    inline T& operator[](size_t index) const {
+        crit(index < used, "array out of bound");
+        return array[index];
+    }
+
+    inline T* operator()(size_t index) const {
+        crit(index < used, "array out of bound");
+        return &array[index];
+    }
+
+    inline void operator()(size_t index, const T value) {
+        crit(index < used, "array out of bound");
+        array[index] = value;
+    }
+
+    inline T& value(size_t index) const {
+        crit(index < used, "array out of bound");
+        return array[index];
+    }
+
+    inline void value(size_t index, const T value) {
+        crit(index < used, "array out of bound");
+        array[index] = value;
+    }
+};
+
+template<>
+class temporary<char *>
+{
+private:
+    inline temporary(const temporary<char *>&) {};
+
+protected:
+    char *object;
+    size_t used;
+
+public:
+    /**
+     * Construct a temporary object, create our stack frame reference.
+     */
+    inline temporary(size_t size) {
+        object = (char *)::malloc(size);
+        used = size;
+    }
+
+    inline operator char *() const {
+        return object;
+    }
+
+    inline size_t size() const {
+        return used;
+    }
+
+    /**
+     * Access heap object through our temporary directly.
+     * @return reference to heap resident object.
+     */
+    inline char *operator*() const {
         return object;
     }
 
@@ -340,98 +412,76 @@ public:
         return object == NULL;
     }
 
+    inline void release() {
+        if(object) {
+            ::free(object);
+            object = NULL;
+        }
+    }
+
     inline ~temporary() {
-        if(object)
-            delete object;
-        object = NULL;
+        if(object) {
+            ::free(object);
+            object = NULL;
+        }
     }
 };
 
-/**
- * Manage temporary array stored on the heap.   This is used to create an
- * array on the heap who's scope is controlled by the scope of a member
- * function call.  Sometimes we have data types and structures which cannot
- * themselves appear as auto variables.  We may also have a limited stack
- * frame size in a thread context, and yet have a dynamic object that we
- * only want to exist during the life of the method call.  Using temporary
- * allows any type to be created from the heap but have a lifespan of a
- * method's stack frame.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <typename T>
-class temp_array
+template<>
+class temporary<uint8_t *>
 {
+private:
+    inline temporary(const temporary<uint8_t *>&) {};
+
 protected:
-    T *array;
-    size_t size;
+    uint8_t *object;
+    size_t used;
 
 public:
     /**
      * Construct a temporary object, create our stack frame reference.
      */
-    inline temp_array(size_t s) {
-        array =  new T[s]; size = s;
+    inline temporary(size_t size) {
+        object = (uint8_t *)::malloc(size);
+        used = size;
+    }
+
+    inline operator uint8_t *() const {
+        return object;
+    }
+
+    inline size_t size() const {
+        return used;
     }
 
     /**
-     * Construct a temporary object with a copy of some initial value.
-     * @param initial object value to use.
+     * Access heap object through our temporary directly.
+     * @return reference to heap resident object.
      */
-    inline temp_array(const T& initial, size_t s) {
-        array = new T[s];
-        size = s;
-        for(size_t p = 0; p < s; ++p)
-            array[p] = initial;
-    }
-
-    inline void reset(size_t s) {
-        delete[] array; array = new T[s]; size = s;
-    }
-
-    inline void reset(const T& initial, size_t s) {
-        if(array)
-            delete[] array;
-        array = new T[s];
-        size = s;
-        for(size_t p = 0; p < s; ++p)
-            array[p] = initial;
-    }
-
-    inline void set(const T& initial) {
-        for(size_t p = 0; p < size; ++p)
-            array[p] = initial;
-    }
-
-    /**
-     * Disable copy constructor.
-     */
-    temp_array(const temp_array<T>&) {
-        ::abort();
+    inline uint8_t *operator*() const {
+        return object;
     }
 
     inline operator bool() const {
-        return array != NULL;
+        return object != NULL;
     }
 
     inline bool operator!() const {
-        return array == NULL;
+        return object == NULL;
     }
 
-    inline ~temp_array() {
-        if(array)
-            delete[] array;
-        array = NULL;
-        size = 0;
+    inline void release() {
+        if(object) {
+            ::free(object);
+            object = NULL;
+        }
     }
 
-    inline T& operator[](size_t offset) const {
-        crit(offset < size, "array out of bound");
-        return array[offset];
-    }
-
-    inline T* operator()(size_t offset) const {
-        crit(offset < size, "array out of bound");
-        return &array[offset];
+    inline ~temporary() {
+        if(object) {
+            ::free(object);
+            object = NULL;
+        }
     }
 };
 
